import {
    Button, Modal, ModalService, Select, Tabs, TextInput, Text, LoadingService, Form, Row, Textarea, Frame, DialogService, Footer,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdDropdown } from '@zeedhi/vuetify';

export class EscopoFaturamentoController {
    public tekgridEscopoFaturamentoVisible: boolean = true;

    public tabsEscopoFaturamento : boolean = true;

    public vizualizacaoTabsServicosEscopo : boolean = false;

    public vizualizacaoTabsDescricaoEscopo : boolean = false;

    public get getTabsEscopoFaturamento() {
        if (this.tabsEscopoFaturamento) {
            this.setDisplayFlex('container-tela-escopo-faturamento');
        }
        return this.tabsEscopoFaturamento;
    }

    public get getVizualizacaoTabsServicosEscopo() {
        if (this.vizualizacaoTabsServicosEscopo) {
            this.setDisplayFlex('frameServicoEscopo');
        }
        return this.vizualizacaoTabsServicosEscopo;
    }

    public get getVizualizacaoTabsDescricaoEscopo() {
        if (this.vizualizacaoTabsDescricaoEscopo) {
            this.setDisplayFlex('frameDescricaoEscopo');
        }
        return this.vizualizacaoTabsDescricaoEscopo;
    }

    public currentRow: any = {};

    public currentRowGridServEscp: any = {};

    public currentRowGridDescEscp: any = {};

    private basicModal!: Modal;

    private ModalEspFatCompetencia!: Modal;

    get tabsItemEscopoFaturamentoVisible() {
        return !this.tekgridEscopoFaturamentoVisible;
    }

    public openFilterEscopoFaturamento({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;

        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    get textFrameEscopoFaturamento() {
        return this.currentRow.NMTIPOORCAMENTO;
    }

    get textFrameServicoEscopo() {
        return `${this.currentRowGridServEscp.NMSERVORCA} - << ${this.currentRow.NMTIPOORCAMENTO}`;
    }

    get textFrameDescricaoEscopo() {
        return `${this.currentRowGridDescEscp.DSNOMEAGRUPAMENTO} - << ${this.currentRow.NMTIPOORCAMENTO}`;
    }

    public voltarTekGridEscopoFaturamento() {
        this.tekgridEscopoFaturamentoVisible = true;
    }

    public voltarPaginaGridServicoEscopo() {
        const footerSalvarServEscp = Metadata.getInstance<Footer>('footer-salvar-servicosEscopoTab');
        const footerSalvarServEscpParametrosFaturamento = Metadata.getInstance<Footer>('footer-servEscParametrosFaturamento');
        footerSalvarServEscp.isVisible = false;
        footerSalvarServEscpParametrosFaturamento.isVisible = false;
        this.vizualizacaoTabsServicosEscopo = false;
        this.tabsEscopoFaturamento = true;
        this.recarregaFormServEscTabParamFat();
    }

    public recarregaFormServEscTabParamFat() {
        Metadata.getInstance<Frame>('servicoEscopoTabParametrosFaturamento').reload();
        this.desbilitaBotoesFooterServEsc();
        this.setDadosServEscTabParamFat();
    }

    public voltarPaginaGridDescricaoEscopo() {
        this.tabsEscopoFaturamento = true;
        this.vizualizacaoTabsDescricaoEscopo = false;
    }

    public openFrameEscopoFaturamento({ row }: any) {
        this.tekgridEscopoFaturamentoVisible = false;
        this.currentRow = row;
        this.loadTabsEscopoFaturamento();
    }

    // Novo Escopo de Faturamento
    public async abrirModalAdicionaEscopoFaturamento() {
        await this.exibirModalEscopoFaturamento();
        Metadata.getInstance<Text>('titleText').text = 'Novo Escopo de Faturamento';
    }

    public async exibirModalEscopoFaturamento() {
        this.basicModal = await ModalService.createFromJson('escopoFaturamento/modal/escopoFaturamentoModalAddEscopoFaturamento', true);
        this.basicModal.show();
    }

    public fecharModalEscopoFaturamento() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public saveAddEscopoFaturamento() {
        const formModalAddEscopoFaturamento = Metadata.getInstance<Form>('form_modal_adiciona_escopo_faturamento');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-faturamento_grid');
        const params = {
            NMTIPOORCAMENTO: Metadata.getInstance<Select>('modalAddEscopoFaturamento-NMTIPOORCAMENTO').value,
            IDATIVO: Metadata.getInstance<Select>('modalAddEscopoFaturamento-IDATIVO').value,
            IDTIPOORCA: Metadata.getInstance<Select>('modalAddEscopoFaturamento-IDTIPOORCA').value,
            IDALTQTDMEDIDAPROP: Metadata.getInstance<Select>('modalAddEscopoFaturamento-IDALTQTDMEDIDAPROP').value,
            IDALTVRTABRPECPROP: Metadata.getInstance<Select>('modalAddEscopoFaturamento-IDALTVRTABRPECPROP').value,
            IDUTILIZAVARPRECO: Metadata.getInstance<Select>('modalAddEscopoFaturamento-IDUTILIZAVARPRECO').value,
            IDFILTRASERVICO: Metadata.getInstance<Select>('modalAddEscopoFaturamento-IDFILTRASERVICO').value,
        };

        if (Utilities.validateForm(formModalAddEscopoFaturamento)) {
            Http.post('/saveAddEscopoFaturamento', params).then(() => {
                grid.datasource.get();
                this.fecharModalEscopoFaturamento();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageEscopoFaturamento({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteEscopoFaturamento();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteEscopoFaturamento() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-faturamento_grid');
        LoadingService.show();
        Http.post('/deleteEscopoFaturamento', grid.selectedRows)
            .then(() => {
                grid.selectedRows = [];
                grid.reload();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Exclusão não permitida!', 'O registro possui registros filhos.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
    }

    // Carrega as informações de acordo com o item de Escopo de Faturamento selecionado.
    public loadTabsEscopoFaturamento() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabsInformationEscopoFaturamento');
        const footerSalvarInfoEscopoFaturamento = Metadata.getInstance<Footer>('footer-salvar-informacoes-EscopoFaturamento');
        const footerEscFatparametrosFaturamento = Metadata.getInstance<Footer>('footer-EscFatparametrosFaturamento');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabEscopoFaturamento') {
            footerSalvarInfoEscopoFaturamento.isVisible = true;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.carregarDadosEscopoFaturamentoTabEscopoFaturamento();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabTabelaPreco') {
            footerSalvarInfoEscopoFaturamento.isVisible = false;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.setFilterEscopoFaturamentoTabTabelaPreco();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabServicosEscopo') {
            footerSalvarInfoEscopoFaturamento.isVisible = false;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.setFilterEscopoFaturamentoTabServicosEscopo();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabParametrosFaturamento') {
            footerSalvarInfoEscopoFaturamento.isVisible = false;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.setDadosCapaParametrosFaturamento();
            this.setParametrosFaturamentoTabCompetencia();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabDescricaoEscopo') {
            footerSalvarInfoEscopoFaturamento.isVisible = false;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.setFilterEscopoFaturamentoTabDescricaoEscopo();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabIndicesFinanceirosPadroes') {
            footerSalvarInfoEscopoFaturamento.isVisible = false;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.setFilterEscopoFaturamentoTabIndicesFinanceirosPadroes();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_escopoFaturamentoTabAgrupamentosEscopo') {
            footerSalvarInfoEscopoFaturamento.isVisible = false;
            footerEscFatparametrosFaturamento.isVisible = false;
            this.setFilterEscopoFaturamentoTabAgrupamentosEscopo();
        }
    }

    public carregarDadosEscopoFaturamentoTabEscopoFaturamento() {
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-NMTIPOORCAMENTO').value = this.currentRow.NMTIPOORCAMENTO;
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-IDATIVO').value = this.currentRow.IDATIVO;
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-IDTIPOORCA').value = this.currentRow.IDTIPOORCA;
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-IDALTQTDMEDIDAPROP').value = this.currentRow.IDALTQTDMEDIDAPROP;
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-IDALTVRTABRPECPROP').value = this.currentRow.IDALTVRTABRPECPROP;
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-IDUTILIZAVARPRECO').value = this.currentRow.IDUTILIZAVARPRECO;
        Metadata.getInstance<TextInput>('escopoFaturamentoTab-IDFILTRASERVICO').value = this.currentRow.IDFILTRASERVICO;
    }

    public editEscopoFaturamento() {
        const formEscopoFaturamento = Metadata.getInstance<Form>('escopoFaturamentoTab-formRowDadosEscopoFaturamento');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-faturamento_grid');
        const params = {
            NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            NMTIPOORCAMENTO: Metadata.getInstance<Select>('escopoFaturamentoTab-NMTIPOORCAMENTO').value,
            IDATIVO: Metadata.getInstance<Select>('escopoFaturamentoTab-IDATIVO').value,
            IDTIPOORCA: Metadata.getInstance<Select>('escopoFaturamentoTab-IDTIPOORCA').value,
            IDALTQTDMEDIDAPROP: Metadata.getInstance<Select>('escopoFaturamentoTab-IDALTQTDMEDIDAPROP').value,
            IDALTVRTABRPECPROP: Metadata.getInstance<Select>('escopoFaturamentoTab-IDALTVRTABRPECPROP').value,
            IDUTILIZAVARPRECO: Metadata.getInstance<Select>('escopoFaturamentoTab-IDUTILIZAVARPRECO').value,
            IDFILTRASERVICO: Metadata.getInstance<Select>('escopoFaturamentoTab-IDFILTRASERVICO').value,
        };

        if (Utilities.validateForm(formEscopoFaturamento)) {
            Http.post('/editEscopoFaturamento', params).then(() => {
                grid.datasource.get();
                this.voltarTekGridEscopoFaturamento();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    // Envia o identificador do Escopo de Faturamento selecionado para montar os dados relacionados ao GRID da TAB Tabelas de Preço.
    public setFilterEscopoFaturamentoTabTabelaPreco() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-tabela-preco');
            grid.datasource.setFilter({
                NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    // Envia o identificador do Escopo de Faturamento selecionado para montar os dados relacionados ao GRID da TAB Serviço do Escopo
    public setFilterEscopoFaturamentoTabServicosEscopo() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-servico-escopo');
            grid.datasource.setFilter({
                NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    // Envia o identificador do Escopo de Faturamento selecionado para montar os dados relacionados ao GRID da Descrição do Escopo
    public setFilterEscopoFaturamentoTabDescricaoEscopo() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-descricao-escopo');
            grid.datasource.setFilter({
                NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    // Envia o identificador do Escopo de Faturamento selecionado para montar os dados relacionados ao GRID da TAB de Indices Financeiros Padroes.
    public setFilterEscopoFaturamentoTabIndicesFinanceirosPadroes() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-indices-escopo-faturamento');
            grid.datasource.setFilter({
                NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    // Envia o identificador do Escopo de Faturamento selecionado para montar os dados relacionados ao GRID da TAB de Agrupamentos de Escopo.
    public setFilterEscopoFaturamentoTabAgrupamentosEscopo() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-agrupamento-escopo');
            grid.datasource.setFilter({
                NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    // Os métodos abaixo são relacionados a TAB Serviço de Escopo após selecionar um item de serviço.
    public vizualizarFrameServicoEscopo({ row }: any) {
        this.tabsEscopoFaturamento = false;
        this.vizualizacaoTabsServicosEscopo = true;
        this.currentRowGridServEscp = row;
        this.loadTabsServicoEscopo();
    }

    public loadTabsServicoEscopo() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabsInformationServicoEscopo');
        const footerSalvarServEscp = Metadata.getInstance<Footer>('footer-salvar-servicosEscopoTab');
        const footerSalvarServEscpParametrosFaturamento = Metadata.getInstance<Footer>('footer-servEscParametrosFaturamento');
        
        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_servicoEscopoTabServicosEscopo') {
            footerSalvarServEscp.isVisible = true;
            footerSalvarServEscpParametrosFaturamento.isVisible = false;
            this.carregaDadosServicoEscopoTabServicosEscopo();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_servicoEscopoTabParametrosFaturamento') {
            footerSalvarServEscp.isVisible = false;
            footerSalvarServEscpParametrosFaturamento.isVisible = true;
            this.setDadosServEscTabParamFat();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_servicoEscopoTabCompetencia') {
            footerSalvarServEscp.isVisible = false;
            footerSalvarServEscpParametrosFaturamento.isVisible = false;
            this.setFilterServicoEscopoTabCompetencia();
        }
    }

    // Carrega os dados do Item do Serviço do Escopo selecionado.
    public carregaDadosServicoEscopoTabServicosEscopo() {
        Metadata.getInstance<Select>('servicosEscopoTab-NRITESCOPOPROPOSTA').value = this.currentRowGridServEscp.NRITESCOPOPROPOSTA;
        Metadata.getInstance<Select>('servicosEscopoTab-CDSERVORCA').value = this.currentRowGridServEscp.CDSERVORCA;
        Metadata.getInstance<Select>('servicosEscopoTab-IDPREVCOBRFUT').value = this.currentRowGridServEscp.IDPREVCOBRFUT;
    }

    public setDadosServEscTabParamFat() {
        const nritescopoproposta = this.currentRowGridServEscp.NRITESCOPOPROPOSTA;
        LoadingService.show();
        Http.post('/getParamFatItEsc', { NRITESCOPOPROPOSTA: nritescopoproposta }).then((response: IDictionary) => {
            const { data } = response;
            if (data.data.length > 0) {
                this.carregaDadosServEscTabParamFat(data);
            } else {
                this.carregaPadraoTabEscTabParamFat();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregaPadraoTabEscTabParamFat() {
        const actionsServicoEscopo = Metadata.getInstance<ZdDropdown>('dropdown-servicoEscopoTabParametrosFaturamento');
        
        // Habilita opção para adicionar um nova parametro de faturamento
        actionsServicoEscopo.children[0].disabled = false;

        // Desabilita opções de Editar e Excluir
        actionsServicoEscopo.children[1].disabled = true;
        actionsServicoEscopo.children[2].disabled = true;

        Metadata.getInstance<Tabs>('tabsInformationServicoEscopo').tabs[2].disabled = true;
        Metadata.getInstance<Tabs>('tabsInformationServicoEscopo').tabs[2].isVisible = false;
    }

    public carregaDadosServEscTabParamFat(data : any) {
        const actionsServicoEscopo = Metadata.getInstance<ZdDropdown>('dropdown-servicoEscopoTabParametrosFaturamento');
        const idtpfatura = data.data[0].IDTPFATURA;
        
        // Desabilita a opção para adicionar um nova Parametro de Lançamento
        actionsServicoEscopo.children[0].disabled = true;

        // Habilita opções de editar e excluir o lançamento do Parametro de Lançamento
        actionsServicoEscopo.children[1].disabled = false;
        actionsServicoEscopo.children[2].disabled = false;

        if (idtpfatura !== 'P') {
            Metadata.getInstance<Tabs>('tabsInformationServicoEscopo').tabs[2].disabled = false;
            Metadata.getInstance<Tabs>('tabsInformationServicoEscopo').tabs[2].isVisible = true;
        }

        // Informações básicas
        Metadata.getInstance<Select>('parametroFaturamentoTab-NRPARAMFATITESC').value = data.data[0].NRPARAMFATITESC;
        Metadata.getInstance<Select>('parametroFaturamentoTab-IDTPFATURA').value = idtpfatura;
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDTPMEDIDAVARIAVEL').value = data.data[0].CDTPMEDIDAVARIAVEL;
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDPRODUTO').value = data.data[0].CDPRODFATURA;
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDFILIAL').value = data.data[0].CDFILIALFATURA;
        Metadata.getInstance<Select>('parametroFaturamentoTab-NRPARCNEGOCIO').value = data.data[0].NRPARCFATURA;
        
        // Dados Vencimento
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASFATU').value = data.data[0].QTPARCELASFATU;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASAPURACAO').value = data.data[0].QTDIASAPURACAO;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-IDTPCALCVENC').value = data.data[0].IDTPCALCVENC;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIACALCVENC').value = data.data[0].NRDIACALCVENC;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-IDMESCALCVENC').value = data.data[0].IDMESCALCVENC;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASFATUVENC').value = data.data[0].QTDIASFATUVENC;
    
        // Dados Nota Fiscal
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDTIPOOPER').value = data.data[0].CDTIPOOPERFATURA;
        setTimeout(() => {
            Metadata.getInstance<Select>('parametroFaturamentoTab-CDSERIE').value = data.data[0].CDSERIENF;
        }, 0);
        Metadata.getInstance<Textarea>('parametroFaturamentoTab-DSSERVORCANF').value = data.data[0].DSSERVORCANF;
        Metadata.getInstance<Textarea>('parametroFaturamentoTab-DSSERVORCABOL').value = data.data[0].DSSERVORCABOL;

        // Dados Parcelamento
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASPARAUTO').value = data.data[0].QTPARCELASPARAUTO;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASINTERVALOPARAUTO').value = data.data[0].QTDIASINTERVALOPARAUTO;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIAVENCIMENTOPARAUTO').value = data.data[0].NRDIAVENCIMENTOPARAUTO;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTCARENCIADIASPARAUTO').value = data.data[0].QTCARENCIADIASPARAUTO;
    }

    public habilitaCamposServEscEspFat() {
        Metadata.getInstance<Select>('parametroFaturamentoTab-IDTPFATURA').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDPRODUTO').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDFILIAL').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoTab-NRPARCNEGOCIO').readonly = false;
        
        // Dados Vencimento
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASFATU').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASAPURACAO').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-IDTPCALCVENC').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIACALCVENC').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-IDMESCALCVENC').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASFATUVENC').readonly = false;
    
        // Dados Nota Fiscal
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDTIPOOPER').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoTab-CDSERIE').readonly = false;
        Metadata.getInstance<Textarea>('parametroFaturamentoTab-DSSERVORCANF').readonly = false;
        Metadata.getInstance<Textarea>('parametroFaturamentoTab-DSSERVORCABOL').readonly = false;
    }

    public adicionarServEscTabParamFat() {
        const actionsServEscTabParamFat = Metadata.getInstance<ZdDropdown>('dropdown-servicoEscopoTabParametrosFaturamento');

        this.habilitaCamposServEscEspFat();
        this.habilitaBotoesFooterServEsc();

        // Desabilita o botão adicionar
        actionsServEscTabParamFat.children[0].disabled = true;
    }

    public editarServEscTabParamFat() {
        const actionsServEscTabParamFat = Metadata.getInstance<ZdDropdown>('dropdown-servicoEscopoTabParametrosFaturamento');

        this.habilitaCamposServEscEspFat();
        this.habilitaBotoesFooterServEsc();

        // Desabilita o botão editar
        actionsServEscTabParamFat.children[1].disabled = true;
    }

    public desbilitaBotoesFooterServEsc() {
        const cancelButton = Metadata.getInstance<Button>('cancelButton-servEscParametrosFaturamento');
        const saveButton = Metadata.getInstance<Button>('saveButton-servEscParametrosFaturamento');

        // desabilita os botões do footer
        cancelButton.disabled = true;
        saveButton.disabled = true;
    }

    public habilitaBotoesFooterServEsc() {
        const cancelButton = Metadata.getInstance<Button>('cancelButton-servEscParametrosFaturamento');
        const saveButton = Metadata.getInstance<Button>('saveButton-servEscParametrosFaturamento');

        // Habilita os botões do footer
        cancelButton.disabled = false;
        saveButton.disabled = false;
    }

    public saveServEscTabParamFat() {
        const formRowDadosParametroFaturamento = Metadata.getInstance<Form>('parametroFaturamentoTab-formRowDadosParametroFaturamento');
        const formEditDadosVencimento = Metadata.getInstance<Form>('parametroFaturamentoTab-formEditDadosVencimento');
        const formEditNotaFiscal = Metadata.getInstance<Form>('parametroFaturamentoTab-formEditNotaFiscal');
        const formEditDadosParcelamento = Metadata.getInstance<Form>('parametroFaturamentoTab-formEditDadosParcelamento');
        const cdfilial = Metadata.getInstance<Select>('parametroFaturamentoTab-CDFILIAL');
        const cdserie = Metadata.getInstance<Select>('parametroFaturamentoTab-CDSERIE');

        const params = {
            NRPARAMFATITESC: Metadata.getInstance<Select>('parametroFaturamentoTab-NRPARAMFATITESC').value,
            NRITESCOPOPROPOSTA: this.currentRowGridServEscp.NRITESCOPOPROPOSTA,
            NRTIPOORCAMENTO: this.currentRowGridServEscp.NRTIPOORCAMENTO,
            CDPRODFATURA: Metadata.getInstance<Select>('parametroFaturamentoTab-CDPRODUTO').value,
            CDFILIALFATURA: (cdfilial.value === null) ? null : cdfilial.selectValue.CDFILIAL,
            NRPARCFATURA: Metadata.getInstance<Select>('parametroFaturamentoTab-NRPARCNEGOCIO').value,
            IDTPFATURA: Metadata.getInstance<Select>('parametroFaturamentoTab-IDTPFATURA').value,
            QTPARCELASFATU: Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASFATU').value,
            QTDIASAPURACAO: Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASAPURACAO').value,
            NRDIACALCVENC: Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIACALCVENC').value,
            QTDIASFATUVENC: Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASFATUVENC').value,
            IDTPCALCVENC: Metadata.getInstance<TextInput>('parametroFaturamentoTab-IDTPCALCVENC').value,
            IDMESCALCVENC: Metadata.getInstance<TextInput>('parametroFaturamentoTab-IDMESCALCVENC').value,
            CDTIPOOPERFATURA: Metadata.getInstance<Select>('parametroFaturamentoTab-CDTIPOOPER').value,
            CDEMPRESA: (cdfilial.value === null) ? null : cdfilial.selectValue.CDEMPRESA,
            CDSERIENF: (cdserie.value === null) ? null : cdserie.selectValue.CDSERIE,
            CDINSCESTA: (cdserie.value === null) ? null : cdserie.selectValue.CDINSCESTA,
            CDINSCMUNI: (cdserie.value === null) ? null : cdserie.selectValue.CDINSCMUNI,
            CDTPMEDIDAVARIAVEL: Metadata.getInstance<Select>('parametroFaturamentoTab-CDTPMEDIDAVARIAVEL').value,
            DSSERVORCABOL: Metadata.getInstance<Textarea>('parametroFaturamentoTab-DSSERVORCABOL').value,
            DSSERVORCANF: Metadata.getInstance<Textarea>('parametroFaturamentoTab-DSSERVORCANF').value,
            QTPARCELASPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTPARCELASPARAUTO').value,
            QTDIASINTERVALOPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTDIASINTERVALOPARAUTO').value,
            NRDIAVENCIMENTOPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoTab-NRDIAVENCIMENTOPARAUTO').value,
            QTCARENCIADIASPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoTab-QTCARENCIADIASPARAUTO').value,
        };

        if (Utilities.validateForm(formRowDadosParametroFaturamento) && Utilities.validateForm(formEditDadosVencimento) && Utilities.validateForm(formEditNotaFiscal) && Utilities.validateForm(formEditDadosParcelamento)) {
            LoadingService.show();
            Http.post('/saveServEscTabParamFat', params).then(() => {
                this.recarregaFormServEscTabParamFat();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'Já existe um Parâmetro de Faturamento cadastrado para este Item.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageServEscTabParamFat({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir os Parâmetros de Faturamento deste item?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteServEscTabParamFat();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteServEscTabParamFat() {
        const params = {
            NRITESCOPOPROPOSTA: this.currentRowGridServEscp.NRITESCOPOPROPOSTA,
        };
        
        LoadingService.show();
        Http.post('/deleteServEscTabParamFat', params)
            .then(() => {
                this.recarregaFormServEscTabParamFat();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }
    
    // Envia o identificador do Item de Escopo para montar os dados relacionados a TAB Competência dos Parametros de Escopo.
    public setFilterServicoEscopoTabCompetencia() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-servicos-escopo-competencia');
            grid.datasource.setFilter({
                NRITESCOPOPROPOSTA: this.currentRowGridServEscp.NRITESCOPOPROPOSTA,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    // Métodos relacionados a TAB de Descrição de Escopo
    public vizualizarFrameDescricaoEscopo({ row }: any) {
        this.vizualizacaoTabsDescricaoEscopo = true;
        this.tabsEscopoFaturamento = false;
        this.currentRowGridDescEscp = row;
        this.loadTabsDescricaoEscopo();
    }

    public loadTabsDescricaoEscopo() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabsInformationDescricaoEscopo');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_DescricaoEscopoTabDescricaoEscopo') {
            this.carregarDadosEscopoFaturamentoTabDescricaoEscopo();
        }
    }

    public carregarDadosEscopoFaturamentoTabDescricaoEscopo() {
        Metadata.getInstance<Select>('descricaoEscopoTab-NRDESCAGRUPESCOPO').value = this.currentRowGridDescEscp.NRDESCAGRUPESCOPO;
        Metadata.getInstance<TextInput>('descricaoEscopoTab-DSNOMEAGRUPAMENTO').value = this.currentRowGridDescEscp.DSNOMEAGRUPAMENTO;
        Metadata.getInstance<Select>('descricaoEscopoTab-IDATIVO').value = this.currentRowGridDescEscp.IDATIVO;
        Metadata.getInstance<Select>('descricaoEscopoTab-IDTPFATURA').value = this.currentRowGridDescEscp.IDTPFATURA;
        Metadata.getInstance<Select>('descricaoEscopoTab-CDEMPRESA').value = this.currentRowGridDescEscp.CDEMPRESA;
        Metadata.getInstance<Select>('descricaoEscopoTab-NRPARCNEGOCIO').value = this.currentRowGridDescEscp.NRPARCNEGOCIO;
        Metadata.getInstance<Select>('descricaoEscopoTab-CDTIPOOPER').value = this.currentRowGridDescEscp.CDTIPOOPER;
        setTimeout(() => {
            Metadata.getInstance<Select>('descricaoEscopoTab-CDSERIE').value = this.currentRowGridDescEscp.CDSERIENF;
        }, 0);
        Metadata.getInstance<Select>('descricaoEscopoTab-IDPREVCOBRFUT').value = this.currentRowGridDescEscp.IDPREVCOBRFUT;
    }

    // Os métodos abaixo são relacionados aos Parametros de Faturamento.
    public loadTabsCapaParametrosFaturamento() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabEscFatParametroFaturamentoTab');
        const footerSalvarCapaParametrosFaturamento = Metadata.getInstance<Footer>('footer-EscFatparametrosFaturamento');
        
        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_parametro_faturamento_dados_gerais') {
            footerSalvarCapaParametrosFaturamento.isVisible = true;
            this.setDadosCapaParametrosFaturamento();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_parametro_faturamento_competencia') {
            footerSalvarCapaParametrosFaturamento.isVisible = false;
            this.setParametrosFaturamentoTabCompetencia();
        }
    }
    
    public recarregaFormCapaParametrosFaturamento() {
        Metadata.getInstance<Frame>('tab_parametro_faturamento_dados_gerais').reload();
        this.desbilitaBotoesFooterCapa();
        this.setDadosCapaParametrosFaturamento();
    }

    public setDadosCapaParametrosFaturamento() {
        const footerEscFatparametrosFaturamento = Metadata.getInstance<Footer>('footer-EscFatparametrosFaturamento');
        footerEscFatparametrosFaturamento.isVisible = true;

        const nrtipoorcamento = this.currentRow.NRTIPOORCAMENTO;
        LoadingService.show();
        Http.post('/getCapaParamFatEsc', { NRTIPOORCAMENTO: nrtipoorcamento }).then((response: IDictionary) => {
            const { data } = response;
            if (data.data.length > 0) {
                this.carregaDadosCapaParametrosFaturamento(data);
            } else {
                this.carregaPadraoCapaParametrosFaturamento();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregaPadraoCapaParametrosFaturamento() {
        const actionsDadosGerais = Metadata.getInstance<ZdDropdown>('dropdown-parametroFaturamentoDadosGerais');
        
        // Habilita opção para adicionar um nova parametro de faturamento
        actionsDadosGerais.children[0].disabled = false;

        // Desabilita opções de Editar e Excluir
        actionsDadosGerais.children[1].disabled = true;
        actionsDadosGerais.children[2].disabled = true;

        Metadata.getInstance<Tabs>('tabEscFatParametroFaturamentoTab').tabs[1].disabled = true;
        Metadata.getInstance<Tabs>('tabEscFatParametroFaturamentoTab').tabs[1].isVisible = false;
    }

    public carregaDadosCapaParametrosFaturamento(data : any) {
        const actionsDadosGerais = Metadata.getInstance<ZdDropdown>('dropdown-parametroFaturamentoDadosGerais');
        const idtpfatura = data.data[0].IDTPFATURA;
        
        // Desabilita a opção para adicionar um nova Parametro de Lançamento
        actionsDadosGerais.children[0].disabled = true;

        // Habilita opções de editar e excluir o lançamento do Parametro de Lançamento
        actionsDadosGerais.children[1].disabled = false;
        actionsDadosGerais.children[2].disabled = false;

        if (idtpfatura !== 'P') {
            Metadata.getInstance<Tabs>('tabEscFatParametroFaturamentoTab').tabs[1].disabled = false;
            Metadata.getInstance<Tabs>('tabEscFatParametroFaturamentoTab').tabs[1].isVisible = true;
        }

        // Informações básicas
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRCAPAPARAMFATESC').value = data.data[0].NRCAPAPARAMFATESC;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPFATURA').value = idtpfatura;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDTPMEDIDAVARIAVEL').value = data.data[0].CDTPMEDIDAVARIAVEL;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDPRODUTO').value = data.data[0].CDPRODFATURA;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDFILIAL').value = data.data[0].CDFILIALFATURA;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-NRPARCNEGOCIO').value = data.data[0].NRPARCFATURA;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCPARCINC').value = data.data[0].IDTPCALCPARCINC;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDAGRUPPARC').value = data.data[0].IDAGRUPPARC;
        
        // Dados Vencimento
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASFATU').value = data.data[0].QTPARCELASFATU;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASAPURACAO').value = data.data[0].QTDIASAPURACAO;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-IDTPCALCVENC').value = data.data[0].IDTPCALCVENC;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIACALCVENC').value = data.data[0].NRDIACALCVENC;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-IDMESCALCVENC').value = data.data[0].IDMESCALCVENC;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASFATUVENC').value = data.data[0].QTDIASFATUVENC;
    
        // Dados Nota Fiscal
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDTIPOOPER').value = data.data[0].CDTIPOOPERFATURA;
        setTimeout(() => {
            Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDSERIE').value = data.data[0].CDSERIENF;
        }, 0);
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDMUNIPRESTSERV').value = data.data[0].IDMUNIPRESTSERV;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTIPONATUOPER').value = data.data[0].IDTIPONATUOPER;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDNATUOPER').value = data.data[0].IDNATUOPER;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCDTSAIDANF').value = data.data[0].IDTPCALCDTSAIDANF;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRCALCULODTSAIDANF').value = data.data[0].NRCALCULODTSAIDANF;
        Metadata.getInstance<Textarea>('parametroFaturamentoDadosGeraisTab-DSSERVORCANF').value = data.data[0].DSSERVORCANF;
        Metadata.getInstance<Textarea>('parametroFaturamentoDadosGeraisTab-DSSERVORCABOL').value = data.data[0].DSSERVORCABOL;

        // Dados Parcelamento
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASPARAUTO').value = data.data[0].QTPARCELASPARAUTO;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASINTERVALOPARAUTO').value = data.data[0].QTDIASINTERVALOPARAUTO;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIAVENCIMENTOPARAUTO').value = data.data[0].NRDIAVENCIMENTOPARAUTO;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTCARENCIADIASPARAUTO').value = data.data[0].QTCARENCIADIASPARAUTO;
    }

    public habilitaCamposCapaParametrosFaturamento() {
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPFATURA').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDPRODUTO').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDFILIAL').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-NRPARCNEGOCIO').readonly = false;
        
        // Dados Vencimento
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASFATU').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASAPURACAO').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCVENC').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIACALCVENC').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-IDMESCALCVENC').readonly = false;
        Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASFATUVENC').readonly = false;
    
        // Dados Nota Fiscal
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDTIPOOPER').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDSERIE').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDMUNIPRESTSERV').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTIPONATUOPER').readonly = false;
        Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCDTSAIDANF').readonly = false;
        Metadata.getInstance<Textarea>('parametroFaturamentoDadosGeraisTab-DSSERVORCANF').readonly = false;
        Metadata.getInstance<Textarea>('parametroFaturamentoDadosGeraisTab-DSSERVORCABOL').readonly = false;
    }

    public adicionarCapaParametrosFaturamento() {
        const actionsDadosGerais = Metadata.getInstance<ZdDropdown>('dropdown-parametroFaturamentoDadosGerais');
        
        this.habilitaCamposCapaParametrosFaturamento();
        this.habilitaBotoesFooterCapa();

        // Desabilita o botao adicionar
        actionsDadosGerais.children[0].disabled = true;
    }

    public editarCapaParametrosFaturamento() {
        const actionsDadosGerais = Metadata.getInstance<ZdDropdown>('dropdown-parametroFaturamentoDadosGerais');

        this.habilitaCamposCapaParametrosFaturamento();

        // Desabilita o botao editar
        actionsDadosGerais.children[1].disabled = true;

        // Habilita os botões do footer
        this.habilitaBotoesFooterCapa();
    }

    public desbilitaBotoesFooterCapa() {
        const cancelButton = Metadata.getInstance<Button>('cancelButton-EscFatparametrosFaturamento');
        const saveButton = Metadata.getInstance<Button>('saveButton-EscFatparametrosFaturamento');

        // desabilita os botões do footer
        cancelButton.disabled = true;
        saveButton.disabled = true;
    }

    public habilitaBotoesFooterCapa() {
        const cancelButton = Metadata.getInstance<Button>('cancelButton-EscFatparametrosFaturamento');
        const saveButton = Metadata.getInstance<Button>('saveButton-EscFatparametrosFaturamento');

        // Habilita os botões do footer
        cancelButton.disabled = false;
        saveButton.disabled = false;
    }

    public saveCapaParametrosFaturamento() {
        const formRowParametroFaturamentoDadosGerais = Metadata.getInstance<Form>('formRowParametroFaturamentoDadosGerais');
        const formEscFatEditParamFatDadosGeraisDadosVencimento = Metadata.getInstance<Form>('formEscFatEditParamFatDadosGeraisDadosVencimento');
        const formParamFatDadosGeraisEditNotaFiscal = Metadata.getInstance<Form>('formParamFatDadosGeraisEditNotaFiscal');
        const formParamFatDadosGeraisEditDadosParcelamento = Metadata.getInstance<Form>('formParamFatDadosGeraisEditDadosParcelamento');
        const cdfilial = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDFILIAL');
        const cdserie = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDSERIE');

        const params = {
            NRCAPAPARAMFATESC: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRCAPAPARAMFATESC').value,
            NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            CDPRODFATURA: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDPRODUTO').value,
            CDFILIALFATURA: (cdfilial.value === null) ? null : cdfilial.selectValue.CDFILIAL,
            NRPARCFATURA: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-NRPARCNEGOCIO').value,
            IDTPFATURA: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPFATURA').value,
            QTPARCELASFATU: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASFATU').value,
            IDTPCALCPARCINC: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCPARCINC').value,
            IDAGRUPPARC: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDAGRUPPARC').value,
            QTDIASAPURACAO: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASAPURACAO').value,
            NRDIACALCVENC: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIACALCVENC').value,
            QTDIASFATUVENC: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASFATUVENC').value,
            IDTPCALCVENC: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCVENC').value,
            IDMESCALCVENC: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-IDMESCALCVENC').value,
            CDTIPOOPERFATURA: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDTIPOOPER').value,
            CDEMPRESA: (cdfilial.value === null) ? null : cdfilial.selectValue.CDEMPRESA,
            CDSERIENF: (cdserie.value === null) ? null : cdserie.selectValue.CDSERIE,
            CDINSCESTA: (cdserie.value === null) ? null : cdserie.selectValue.CDINSCESTA,
            CDINSCMUNI: (cdserie.value === null) ? null : cdserie.selectValue.CDINSCMUNI,
            IDTPCALCDTSAIDANF: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCDTSAIDANF').value,
            NRCALCULODTSAIDANF: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRCALCULODTSAIDANF').value,
            IDMUNIPRESTSERV: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDMUNIPRESTSERV').value,
            IDTIPONATUOPER: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTIPONATUOPER').value,
            IDNATUOPER: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDNATUOPER').value,
            CDTPMEDIDAVARIAVEL: Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDTPMEDIDAVARIAVEL').value,
            DSSERVORCABOL: Metadata.getInstance<Textarea>('parametroFaturamentoDadosGeraisTab-DSSERVORCABOL').value,
            DSSERVORCANF: Metadata.getInstance<Textarea>('parametroFaturamentoDadosGeraisTab-DSSERVORCANF').value,
            QTPARCELASPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASPARAUTO').value,
            QTDIASINTERVALOPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASINTERVALOPARAUTO').value,
            NRDIAVENCIMENTOPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIAVENCIMENTOPARAUTO').value,
            QTCARENCIADIASPARAUTO: Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTCARENCIADIASPARAUTO').value,
        };

        if (Utilities.validateForm(formRowParametroFaturamentoDadosGerais) && Utilities.validateForm(formEscFatEditParamFatDadosGeraisDadosVencimento) && Utilities.validateForm(formParamFatDadosGeraisEditNotaFiscal) && Utilities.validateForm(formParamFatDadosGeraisEditDadosParcelamento)) {
            LoadingService.show();
            Http.post('/saveCapaParametrosFaturamento', params).then(() => {
                this.recarregaFormCapaParametrosFaturamento();
                this.setDadosCapaParametrosFaturamento();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'Já existe um Parâmetro de Faturamento cadastrado para este Escopo.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageCapaParametrosFaturamento({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir os Parâmetros de Faturamento deste item?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteCapaParametrosFaturamento();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteCapaParametrosFaturamento() {
        const params = {
            NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
        };
        
        LoadingService.show();
        Http.post('/deleteCapaParametrosFaturamento', params)
            .then(() => {
                this.recarregaFormCapaParametrosFaturamento();
                this.setDadosCapaParametrosFaturamento();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public alteraVisibilidadeFormParametrosFaturamento() {
        const tipoFaturamento = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPFATURA').value;
        const tipoMedidaVariavel = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-CDTPMEDIDAVARIAVEL');
        const rowDadosVencimento = Metadata.getInstance<Row>('rowParamFatDadosGeraisDadosVencimento');
        const formDadosVencimento = Metadata.getInstance<Form>('formEscFatEditParamFatDadosGeraisDadosVencimento');
        const rowDadosParcelamento = Metadata.getInstance<Row>('rowParamFatDadosGeraisEditDadosParcelamento');
        const formDadosParcelamento = Metadata.getInstance<Form>('formParamFatDadosGeraisEditDadosParcelamento');
        const qtdMaxParcelas = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASFATU');
        const qtdDiasApuracao = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASAPURACAO');
        const tpCalcVencimento = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCVENC');
        const diaVencimento = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIACALCVENC');
        const mesCalculoVencimento = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDMESCALCVENC');
        const qtdParcelas = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTPARCELASPARAUTO');
        const qtdIntervaloParcelas = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTDIASINTERVALOPARAUTO');
        const diaVencimentoParcela = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIAVENCIMENTOPARAUTO');
        const tempoCarencia = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-QTCARENCIADIASPARAUTO');
        const tabCompetencia = Metadata.getInstance<Tabs>('tabEscFatParametroFaturamentoTab').tabs[1];

        if (tipoFaturamento === 'X') {
            tipoMedidaVariavel.readonly = false;
        } else {
            tipoMedidaVariavel.readonly = true;
            tipoMedidaVariavel.value = null;
        }

        if (tipoFaturamento === 'P') {
            rowDadosVencimento.isVisible = true;
            formDadosVencimento.isVisible = true;
            rowDadosParcelamento.isVisible = true;
            formDadosParcelamento.isVisible = true;
            qtdMaxParcelas.addValidation('required', {});
            qtdDiasApuracao.addValidation('required', {});
            tpCalcVencimento.addValidation('required', {});
            mesCalculoVencimento.addValidation('required', {});
            diaVencimento.addValidation('required', {});
            qtdParcelas.readonly = false;
            qtdIntervaloParcelas.readonly = false;
            diaVencimentoParcela.readonly = false;
            tempoCarencia.readonly = false;
            tabCompetencia.isVisible = false;
        } else {
            rowDadosVencimento.isVisible = false;
            formDadosVencimento.isVisible = false;
            rowDadosParcelamento.isVisible = false;
            formDadosParcelamento.isVisible = false;
            qtdMaxParcelas.removeValidation('required');
            qtdDiasApuracao.removeValidation('required');
            tpCalcVencimento.removeValidation('required');
            mesCalculoVencimento.removeValidation('required');
            diaVencimento.removeValidation('required');
            qtdParcelas.readonly = true;
            qtdIntervaloParcelas.readonly = true;
            diaVencimentoParcela.readonly = true;
            tempoCarencia.readonly = true;
            qtdParcelas.value = null;
            qtdIntervaloParcelas.value = null;
            diaVencimentoParcela.value = null;
            tempoCarencia.value = null;
            tabCompetencia.isVisible = true;
        }
    }

    public async atualizaNotaFiscalDadosGerais({ component }: IEventParam<Select>) {
        const cdSerieNf = Metadata.getInstance<Select>(`${component.name.split('-')[0]}-CDSERIE`);
        
        if (component.value) {
            cdSerieNf.datasource.setFilter({
                CDEMPRESA: component.selectValue.CDEMPRESA,
            });
        }
        
        Utilities.alertMessage('Atenção: Campo "Série da Nota Fiscal" foi recarregado', 'warning');
    }

    public controlarTipoCalculoVencimentoDadosGerais({ component }: any) {
        const idtpcalcvenc = component.value;
        const diaVencimento = Metadata.getInstance<TextInput>('parametroFaturamentoDadosGeraisTab-NRDIACALCVENC');
        const mesCalculoVencimento = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDMESCALCVENC');
        const diasMinimosFatVenc = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-QTDIASFATUVENC');

        if (idtpcalcvenc === '1') { //  Dia fixo para a data de vencimento
            diaVencimento.label = 'Dia de Vencimento';
            
            mesCalculoVencimento.readonly = false;
            mesCalculoVencimento.addValidation('required', {});
            
            diasMinimosFatVenc.readonly = false;
            diasMinimosFatVenc.addValidation('required', {});
        } else if (idtpcalcvenc === '2') { // Quantidade de dias após a data de emissão
            diaVencimento.label = 'Quantidade de Dias';

            mesCalculoVencimento.readonly = true;
            mesCalculoVencimento.removeValidation('required');
            mesCalculoVencimento.value = '';
            
            diasMinimosFatVenc.readonly = true;
            diasMinimosFatVenc.removeValidation('required');
            mesCalculoVencimento.value = '';
        }
    }

    public alterarVisibilidadefieldNaturezaOperacao({ component } : any) {
        const idnatuoper = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDNATUOPER');

        if (component.value === 'F') {
            idnatuoper.readonly = false;
            idnatuoper.addValidation('required', {});
        } else {
            idnatuoper.readonly = true;
            idnatuoper.removeValidation('required');
            idnatuoper.value = null;
        }
    }

    // Valida o tipo de cálculo da data de saída e habilita o campo do parametro de cálculo.
    public alterarVisibilidadefieldCalculoDtSaida({ component } : any) {
        const nrcalculodtsaidanf = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-NRCALCULODTSAIDANF');

        if (!component.value || component.value === 1) {
            nrcalculodtsaidanf.readonly = true;
            nrcalculodtsaidanf.label = 'Dia fixo / Quantidade de dias para data de saída';
            nrcalculodtsaidanf.removeValidation('required');
            nrcalculodtsaidanf.value = null;
        } else {
            const nmlabel = ([2, 3].indexOf(component.value) !== -1) ? 'Dia fixo para data de saída' : 'Quantidade de dias para data de saída';
            nrcalculodtsaidanf.readonly = false;
            nrcalculodtsaidanf.label = nmlabel;
            nrcalculodtsaidanf.addValidation('required', {});
        }
    }

    // Valida se o valor digitado se aplica ao tipo de calculo escolhido.
    public validarParametrosCalculoDtSaida({ component } : any) {
        const idtpcalcdtsaidanf = Metadata.getInstance<Select>('parametroFaturamentoDadosGeraisTab-IDTPCALCDTSAIDANF');

        if ([2, 3].indexOf(idtpcalcdtsaidanf.value) !== -1) {
            if (component.value > 31) {
                Utilities.alertMessage('Operação bloqueada!<br><br>Não é possível atribuir um dia fixo maior que 31 para o cálculo da data de saída da nota fiscal.', 'error');
                component.value = 0;
            }
        }
    }

    // Os métodos abaixo são relacionados ao Parametro de Faturamento do Escopo na TAB Competência.

    public setParametrosFaturamentoTabCompetencia() {
        try {
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-parametros-faturamento-competencia');
            grid.datasource.setFilter({
                NRTIPOORCAMENTO: this.currentRow.NRTIPOORCAMENTO,
            });
        } catch (e) {
            // console.error(e);
        }
    }

    public async abrirModalAdicionaParamFatCompetencia() {
        await this.exibirModalParamFatCompetencia();
        Metadata.getInstance<Text>('titleText').text = 'Nova Competência';
    }

    public async exibirModalParamFatCompetencia() {
        this.ModalEspFatCompetencia = await ModalService.createFromJson('escopoFaturamento/modal/parametrosFaturamentoModalCompetencia', true);
        this.ModalEspFatCompetencia.show();
    }

    public fecharModalParamFatCompetencia() {
        this.ModalEspFatCompetencia.hide();
        this.ModalEspFatCompetencia?.destroy();
    }

    public async abrirModalEdicaoParamFatCompetencia({ row } : any) {
        await this.exibirModalParamFatCompetencia();
        Metadata.getInstance<Text>('titleText').text = 'Editar Competência';
        Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRPARAMFATESC').value = row.NRPARAMFATESC;
        Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETINI').value = row.NRDIACOMPETINI;
        Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCOMPETINI').value = row.IDMESCOMPETINI;
        Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCOMPETFIN').value = row.IDMESCOMPETFIN;
        Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETFIN').value = row.NRDIACOMPETFIN;
        Metadata.getInstance<TextInput>('paramFatCompetenciaModal-QTDIASAPURACAO').value = row.QTDIASAPURACAO;
        Metadata.getInstance<Select>('paramFatCompetenciaModal-IDTPCALCVENC').value = row.IDTPCALCVENC;
        Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACALCVENC').value = row.NRDIACALCVENC;
        Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCALCVENC').value = row.IDMESCALCVENC;
        Metadata.getInstance<TextInput>('paramFatCompetenciaModal-QTDIASFATUVENC').value = row.QTDIASFATUVENC;
    }

    public saveParamFatCompetencia() {
        const formParamFatComp = Metadata.getInstance<Form>('formModalParametrosFaturamentoCompetencia');
        const formParamFatCompDadosVenc = Metadata.getInstance<Form>('formModalParamFatCompetenciaDadosVencimento');
        const gridEscFat = Metadata.getInstance<TekGrid>('tek-grid-escopo-faturamento_grid');
        const gridParamFatComp = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-parametros-faturamento-competencia');
        const nrtipoorcamento = gridEscFat.datasource.currentRow.NRTIPOORCAMENTO;

        const params = {
            NRPARAMFATESC: Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRPARAMFATESC').value,
            NRTIPOORCAMENTO: nrtipoorcamento,
            NRDIACOMPETINI: Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETINI').value,
            IDMESCOMPETINI: Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCOMPETINI').value,
            IDMESCOMPETFIN: Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCOMPETFIN').value,
            NRDIACOMPETFIN: Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETFIN').value,
            QTDIASAPURACAO: Metadata.getInstance<TextInput>('paramFatCompetenciaModal-QTDIASAPURACAO').value,
            IDTPCALCVENC: Metadata.getInstance<Select>('paramFatCompetenciaModal-IDTPCALCVENC').value,
            NRDIACALCVENC: Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACALCVENC').value,
            IDMESCALCVENC: Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCALCVENC').value,
            QTDIASFATUVENC: Metadata.getInstance<TextInput>('paramFatCompetenciaModal-QTDIASFATUVENC').value,
        };

        if (Utilities.validateForm(formParamFatComp) && Utilities.validateForm(formParamFatCompDadosVenc)) {
            LoadingService.show();
            Http.post('/saveParamFatCompetencia', params).then(() => {
                gridParamFatComp.datasource.get();
                this.fecharModalParamFatCompetencia();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'Já existe um registro para o período de competência informado.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageParamFatCompetencia({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteParamFatCompetencia();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteParamFatCompetencia() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-parametros-faturamento-competencia');
        LoadingService.show();
        Http.post('/deleteParamFatCompetencia', grid.selectedRows)
            .then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    // Os métodos abaixo são relacionados ao Modal de Edição e a adição de Competência para os Parametros de Faturamento.
    public validaCampoVazio(campo : string) : boolean {
        if ((campo === '') || (campo === undefined) || (campo === null)) {
            return true;
        }
        return false;
    }

    public validarPeriodoParamFatCompetencia() {
        const diaInicial = Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETINI').value;
        const diaFinal = Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETFIN').value;
        const mesInicial = Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCOMPETINI').value;
        const mesFinal = Metadata.getInstance<Select>('paramFatCompetenciaModal-IDMESCOMPETFIN').value;

        if (!this.validaCampoVazio(diaInicial) && !this.validaCampoVazio(diaFinal)) {
            if (mesInicial === mesFinal && diaInicial > diaFinal) {
                Utilities.alertMessage('Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final.', 'error');
                Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETINI').value = null;
                Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETFIN').value = null;
            } else if (mesInicial > mesFinal) {
                Utilities.alertMessage('Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final', 'error');
                Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETINI').value = null;
                Metadata.getInstance<TextInput>('paramFatCompetenciaModal-NRDIACOMPETFIN').value = null;
            }
        }
    }

    public setDisplayFlex(componentName: string) {
        setTimeout(() => {
            const a = document.getElementById(componentName) as any;
            a.style.display = 'flex';
        }, 0);
    }
}
